% CoSMoS-COAST Xshore model w/ ensemble waves. SeanV.
close all; clc; %clear all;

% seed random number generator
seed=1;
rng(seed);

% number of ensembles
Nens=1000;
    
% start time and time where ensemble forecast begins
tstart=datenum(1999,1,1);
tforecast1=datenum(2009,1,1);  % after this date, the ensemble forcing is used
tforecast2=datenum(2017,1,1);  % after this date, the ensemble forcing is used
tstop=datenum(2025,1,1);      % the simulation stops at this date

%% Ensemble Wave conditions %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

ENSEMBLE_FORCING=0;

RELOAD_WAVES=1;
if RELOAD_WAVES % if we want to reprocess the waves
    
    load('Wave_hindcast_corrected.mat');
    
    t1=hindcast.time;
    Hs1=hindcast.Hs;
    Tp1=hindcast.tm02;
    Dir1=hindcast.Dir;
    
    load('Results_100IndependentSimulations_3hourly.mat');
    
    Nsim=size(Simulation.Hs,2); % the number of wave ensembles
    
    MEAN_CORRECTION=nanmean(hindcast.Hs)-nanmean(Simulation.Hs(:));
    
    t2=Simulation.time;
    Hs2=Simulation.Hs;
    Tp2=Simulation.Tm;
    Dir2=Simulation.Dir;

    id1=t1>tstart & t1<=tforecast2;
    id2=t2>tforecast2 & t2<=tstop;
    
    t=cat(1,t1(id1),t2(id2));
    
    if ENSEMBLE_FORCING;
        
        if  Nens~=Nsim; % if the number of model ensembles is not the same as the number of wave ensebles
            
            Hs=cat(1,repmat(Hs1(id1),1,Nens),Hs2(id2,randi([1 Nsim],Nens,1)));
            Tp=cat(1,repmat(Tp1(id1),1,Nens),Tp2(id2,randi([1 Nsim],Nens,1)));
            Dir=cat(1,repmat(Dir1(id1),1,Nens),Dir2(id2,randi([1 Nsim],Nens,1)));
            
        else
            
            Hs=cat(1,repmat(Hs1(id1),1,Nens),Hs2(id2,:));
            Tp=cat(1,repmat(Tp1(id1),1,Nens),Tp2(id2,:));
            Dir=cat(1,repmat(Dir1(id1),1,Nens),Dir2(id2,:));
            
        end
        
    else
        
        Hs=cat(1,repmat(Hs1(id1),1,Nens),repmat(Hs2(id2,1),1,Nens));
        Tp=cat(1,repmat(Tp1(id1),1,Nens),repmat(Tp2(id2,1),1,Nens));
        Dir=cat(1,repmat(Dir1(id1),1,Nens),repmat(Dir2(id2,1),1,Nens));
        
    end
    
else
    fprintf('using preivously loaded waves files.\n');
    clearvars -except transects Nens t Hs Tp Dir tstart tstop tforecast1 tforecast2;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% time stepping
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
dt=nanmean(diff(t)); % time step. in [days]

%t=(tstart:dt:tstop)'; % the simulation time
len=length(t);        % the number of time steps

sim_time=NaN(1000,1);

[YEAR,~,~]=datevec(t);
YEAR=unique(YEAR);
NYEARS=length(YEAR);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Set up model initial conditions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
sigmaY=0;
Yst=repmat(sigmaY,1,Nens).*repmat(randn(1,Nens),1,1); % the short-term shoreline component

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% MODEL PARAMETERS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

eps=1e-4;

% a NEW equlibrium shoreline model (modified from Yates et al. 2009)

% DT coefficient  (time scale for accretion)
muDT=28;   % mean of DT+ coefficient
sigmaDT=1; % standard deviation of DT+ coefficient

DT=repmat(muDT,1,Nens)+sigmaDT*repmat(randn(1,Nens),1,1); % Alongshore-uniform, normally distributed ensemble for C

% upper and lower bounds for DT+
DT_max=3*muDT;
DT_min=eps;

DT(DT<DT_min)=DT_min;  % clip DT+ to upper and lower bounds
DT(DT>DT_max)=DT_max;

% DY coefficient (shoreline excursion coefficient)
muDY=10;     % mean of a coefficient
sigmaDY=2;   % standard deviation of a coefficient

DY=repmat(muDY,1,Nens)+sigmaDY*repmat(randn(1,Nens),1,1); % Alongshore-uniform, normally distributed ensemble for a

% upper and lower bounds for a
DY_max=3*muDY;
DY_min=eps;

DY(DY<DY_min)=DY_min;  % clip a to upper and lower bounds
DY(DY>DY_max)=DY_max;

% HSB coefficient (background/average wave height)
muHSB=1.00*nanmean(Hs(:)); % mean of b coefficient
sigmaHSB=0.2*muHSB ; % standard deviation of b coefficient

HSB=repmat(muHSB,1,Nens)+sigmaHSB*repmat(randn(1,Nens),1,1); % Alongshore-uniform, normally distributed ensemble for b

% upper and lower bounds for Hsb
HSB_max=3*muHSB;
HSB_min=eps;

HSB(HSB<HSB_min)=HSB_min;  % clip Hsb to upper and lower bounds
HSB(HSB>HSB_max)=HSB_max;

% long-term rate coefficient
muvlt=0;
vlt=muvlt*ones(1,Nens); % long-term shoreline change rate

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% DATA ASSIMILATION
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

DATA_ASSIMILATION=1; % turn on data assimilation?

% data-assimilation window
DATA_ASSIMILATION_WINDOW=0; % turn on 4-D data assimilation?
T_window=14;                % data-assimilation window [in days]
n_window=T_window/dt;       % data-assimilation window [in the number of time steps]

% modified data assimlation parameters
DT0 =muDT;
DY0 =muDY;
HSB0=muHSB;
vlt0=muvlt;

% widths of the state parameters
sig=0.5;

% new state parameters (modified to preserve expected signs)
DT1 =1/sig*log(DT ./repmat(DT0 ,1,Nens));
DY1 =1/sig*log(DY ./repmat(DY0 ,1,Nens));
HSB1=1/sig*log(HSB./repmat(HSB0,1,Nens));

shoreline_error_magnitude=0.5;
H=[1 0 0 0 0];

P=zeros(length(H)); % initialize error covariance matrix

% Data to assimilate %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% shoreline observations to load
load('Shorecast_complete.mat')

tobs=Shore.time;
Yobs=Shore.average-nanmean(Shore.average);

int=30;
tobs=tobs(1:int:end);
Yobs=Yobs(1:int:end);

Yobs(tobs<tstart)=[];
tobs(tobs<tstart)=[];

Yobs(tobs>tforecast2)=[];
tobs(tobs>tforecast2)=[];

tobs(isnan(Yobs))=[];
Yobs(isnan(Yobs))=[];

SMOOTH=0;
if SMOOTH
    d1 = designfilt('lowpassiir','FilterOrder',4, ...
        'HalfPowerFrequency',0.02,'DesignMethod','butter');
    Ysmooth = filtfilt(d1,Yobs);
    
    %     plot(tobs,Yobs,'b.',tobs,Ysmooth,'-r.');
    %     stop
    
    Yobs=Ysmooth;
end

% average the shorecast data over some period.
AVG_SHORELINES=0;
if AVG_SHORELINES;
    
    period=7; % in days
    
    t1=min(t);
    t2=max(t);
    
    ti=(t1:period:t2)';
    
    len1=length(ti);
    
    Yavg=nan(len1,1);  
    
    for i=1:len1;
        
        id=(tobs>=(ti(i)-period/2) & tobs<(ti(i)+period/2));
        Yavg(i)=nanmean(Yobs(id));
        
    end
    
    ti(isnan(Yavg))=[];
    Yavg(isnan(Yavg))=[];
    
    tobs_final=ti;
    Yobs_final=Yavg;
    
else
    tobs_final=tobs;
    Yobs_final=Yobs;
end

plot(tobs,Yobs,'-b.',tobs_final,Yobs_final,'-r.'); datetick('x');

Yst=repmat(Yobs_final(1),1,Nens);

% data assimilation helper data struct
DA.t=tobs_final; % times of data assimilation
DA.Y=Yobs_final; % shoreline position data to assimilate

idrmda=DA.t<tstart; % remove data from before t0

DA.t(idrmda)=[];
DA.Y(idrmda)=[];

tmp=interp1(t,(1:len)',tobs_final); % find the time steps when data assimilation is to take place
tmp(isnan(tmp))=[];                 % remove NaNs
tmp(tmp>len)=[];                    % remove data assimilation time steps that are larger than the total number of time steps in the simulation
DA.n=sort(floor(tmp));              % time steps when data assimilation is to take place

if (~isempty(DA.n))
    DA.count=1; % initialize counter
    DA.DONE=0;  % and DA is-completed flag
else
    DA.count=0; % initialize counter to zero
    DA.DONE=1;  % and DA is-completed flag to completed (i.e. no data assimilation)
end

DA.nmax=max(DA.n);       % find the maximum data assimilation time step

% noise model for Kalman filter
NOISE=0.2;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% output variables
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

output_interval=8;
len_interval=length(find(mod(1:len-1,output_interval)==0))+1;

t_output=NaN(len_interval,1); t_output(1)=t(1);

YMEAN=NaN(len_interval,1); YMEAN(1,:)=mean(Yst,2);

Y1=NaN(len_interval,1); Y1(1,:)=Yst(1);

YCI1=NaN(len_interval,1); YCI1(1,:)=quantile(Yst,0.025,2);
YCI2=NaN(len_interval,1); YCI2(1,:)=quantile(Yst,0.05,2);
YCI3=NaN(len_interval,1); YCI3(1,:)=quantile(Yst,0.95,2);
YCI4=NaN(len_interval,1); YCI4(1,:)=quantile(Yst,0.975,2);
YMAX=NaN(len_interval,1); YMAX(1,:)=min(Yst);
YY=NaN(len_interval,1,Nens); YY(1,:,:)=Yst;

count_year=1;
YEAR_current=YEAR(1);
YMAX_YEAR=NaN(NYEARS,Nens);

VAR=NaN(len_interval,1); VAR(1,:)=var(Yst);

HSCI1=NaN(len_interval,1); HSCI1(1,:)=quantile(Hs(1,:),0.025,2);
HSCI2=NaN(len_interval,1); HSCI2(1,:)=quantile(Hs(1,:),0.05,2);
HSCI3=NaN(len_interval,1); HSCI3(1,:)=quantile(Hs(1,:),0.95,2);
HSCI4=NaN(len_interval,1); HSCI4(1,:)=quantile(Hs(1,:),0.975,2);

HSMAX =NaN(len_interval,1); HSMAX (1,:)=max(Hs(1,:));
HSMEAN=NaN(len_interval,1); HSMEAN(1,:)=mean(Hs(1,:));
HSVAR =NaN(len_interval,1); HSVAR (1,:)=var(Hs(1,:));

FVAR =NaN(len_interval,1);  FVAR (1,:)=0;

count_output=2;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Set up figure for plotting
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
filename='CoSMoS_COAST_EnKF';

PLOT=0;
PLOT_INTERVAL=8*7*4*5;

MAKE_ANIMATION=1;
if MAKE_ANIMATION % make gif animation
    count=1;
    fps=15;
end

tic2=tic;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% MAIN Simulation (with data assimilation from previous run)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% main time stepping loop
for n=1:len-1
    
    tic
    
    fprintf('COSMOS-COAST: timestep %s. %0.1f%% finished. ',datestr(t(n),'mm/dd/yyyy'),n/len*100); % display progress
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% WAVE MODEL
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    % wave conditions at the current time step
    Hs_now=Hs(n,:,:);
    Tp_now=Tp(n,:,:);
    Dir_now=Dir(n,:,:);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% (FORWARD) SHORELINE MODEL
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    % the equlibrium shoreline position
    Yeq=-repmat(DY0,1,Nens).*exp(sig*DY1).*((Hs_now.^2 - (repmat(HSB0,1,Nens).*exp(sig*HSB1)).^2)./(repmat(HSB0,1,Nens).*exp(sig*HSB1)).^2);
    
    % short-term shoreline change model
    Yst=Yst+dt./(repmat(DT0,1,Nens).*exp(sig*DT1)).*Hs_now./(repmat(HSB0,1,Nens).*exp(sig*HSB1)).*(Yeq-Yst);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % NOISE MODEL
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % add Gaussian Noise to the forward model
%     Yst =Yst +1.0*NOISE*randn(1,Nens);
%     DT1 =DT1 +0.01*NOISE*randn(1,Nens);
%     DY1 =DY1 +0.01*NOISE*randn(1,Nens);
%     HSB1=HSB1+0.01*NOISE*randn(1,Nens);
%     vlt =vlt +0.0*NOISE*randn(1,Nens);
    
    Yst =Yst +1.0*NOISE*randn(1,Nens);
    DT1 =DT1 +0.0*NOISE*randn(1,Nens);
    DY1 =DY1 +0.0*NOISE*randn(1,Nens);
    HSB1=HSB1+0.0*NOISE*randn(1,Nens);
    vlt =vlt +0.0*NOISE*randn(1,Nens);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% (INVERSE) DATA ASSIMILATION MODEL
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if DATA_ASSIMILATION;   % if data assimilation is performed
        
        if (DA.DONE)
            % do nothing
            
        else
            
            [DATA_FOUND,locb1]=ismember(n+1,DA.n); % look for data to assimilate
            
            if any(DATA_FOUND)                  % if data is avaliable to assimilate on the current time step
                
                YOBS_ERROR=shoreline_error_magnitude*randn(Nens,1);
                
                psi_star=[Yst' DT1' DY1' HSB1' vlt']; % get the state vector

                P=cov(psi_star); % calculate covariance amoung ensemble
                
                R1=1/(Nens-1)*YOBS_ERROR'*YOBS_ERROR;
                
                KAL=(P*H')/(H*P*H'+R1);  % compute Kalman gain
                
                YMOD=psi_star*H'; % modeled shoreline position (from PSI_STAR)
                
                % find the time of the last data point within the
                % assimilation window
                
                [MORE_DATA_FOUND,locb2]=ismember((n+2):(n+1+n_window),DA.n); % look for data to assimilate
                
                locb2=locb2(locb2>=1); % get rid of the zeros
                
                if DATA_ASSIMILATION_WINDOW && any(MORE_DATA_FOUND)

                    % run the model until then
                    n_more=find(MORE_DATA_FOUND, 1, 'last');
                    
                    Yst_tmp=NaN(n_more,Nens);
                    
                    Yst_tmp(1,:)=Yst;
                    
                    fprintf('running the model over assimilation window ... ');
                    for nn=1:n_more-1; % run the model over the assimilation window
                        
                        Hs_tmp=Hs(nn,:,:); % get waves for current time step
                        
                        Omega=-Hs_tmp.*((Hs_tmp./(repmat(HSB0,1,Nens).*exp(sig*HSB1))).^2-1+Yst_tmp(nn,:)./(repmat(DY0,1,Nens).*exp(sig*DY1)));
                        
                        % short-term shoreline change model
                        Yst_tmp(nn+1,:)=Yst_tmp(nn,:)+dt./(repmat(DT0 ,1,Nens).*exp(sig*DT1 )).*Omega;
                                                 

                    end
                    fprintf('done.\n');

                    Yobsi=DA.Y(locb2); % get observed data to use in assimilation
                    
                    YMOD=Yst_tmp(MORE_DATA_FOUND,:);
                    
                    shoreline_error=nanmean(repmat(Yobsi,1,Nens)+shoreline_error_magnitude*randn(length(Yobsi),Nens)-YMOD,1)'; % error

                else
                    
                    Yobsi=DA.Y(locb1);                     % get observed data to use in assimilation
                    shoreline_error=Yobsi+YOBS_ERROR-YMOD; % calculate obs - modeled shoreline error
                end
                
                % Kalman filter analysis step
                psi=psi_star+repmat(KAL',Nens,1).*repmat(shoreline_error,1,length(H)); % correct the state vector
                
                % inflation step
%                 lambda=1.01;
%                 psi=repmat(nanmean(psi,1),Nens,1)+lambda*(psi-repmat(nanmean(psi,1),Nens,1));
                
                % calculate  post-data assimilation parameters
                Yst =psi(:,1)';
                DT1 =psi(:,2)';
                DY1 =psi(:,3)';
                HSB1=psi(:,4)';
                vlt =psi(:,5)';
                
                % calculate physical parameters
                DT =DT0 *exp(sig*DT1 );
                DY =DY0 *exp(sig*DY1 );
                HSB=HSB0*exp(sig*HSB1);

                vlt_assimilated=vlt; % save the assimilated long-term rate
                
                if (n+1)>=DA.nmax    % if the counter is greater than or equal to the number of data points to assimilate
                    DA.DONE=1;       % then the data-assimilation process is done for the current transect
                end
                
            end
            
        end
        
        if t(n+1)>tforecast1
            
            DATA_ASSIMILATION=0; % turn off data assimilation
            
            %NOISE=0;
            
            %PLOT_INTERVAL=365*10;  % turn down plotting frequency
            
        end
        
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% OUTPUT
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if mod(n+1,output_interval)==0;
        
        t_output(count_output)=t(n+1);
        
        % calculate annual max
        [YEAR_output,~,~]=datevec(t_output(count_output));
        if YEAR_output>YEAR_current
            count_year=count_year+1;
            YEAR_current=YEAR_output;
        end
        YMAX_YEAR(count_year,:)=min(YMAX_YEAR(count_year,:),Yst);
        
        YMEAN(count_output,:)=mean(Yst,2);           % calculate the ensemble mean
        
        Y1(count_output,:)=Yst(1);           % calculate the ensemble mean
        
        YCI1(count_output,:)=quantile(Yst,0.025);  % and the 95% confidence bands
        YCI2(count_output,:)=quantile(Yst,0.10);   % and the 80% confidence bands
        YCI3(count_output,:)=quantile(Yst,0.90);   % and the 80% confidence bands
        YCI4(count_output,:)=quantile(Yst,0.975);  % and the 95% confidence bands
        
        YMAX(count_output,:)=min(Yst);  % and the 95% confidence bands
        
        VAR(count_output,:)=var(Yst);
        
        HSCI1(count_output,:)=quantile(Hs_now,0.025)-0.0001;  % and the 95% confidence bands
        HSCI2(count_output,:)=quantile(Hs_now,0.10 )-0.001;  % and the 80% confidence bands
        HSCI3(count_output,:)=quantile(Hs_now,0.90 )+0.001;  % and the 80% confidence bands
        HSCI4(count_output,:)=quantile(Hs_now,0.975)-0.0001;  % and the 95% confidence bands
        
        HSMEAN(count_output,:)=nanmean(Hs_now);
        HSMAX(count_output,:)=max(Hs_now);
        HSVAR(count_output,:)=var(Hs_now);

        FVAR(count_output,:)=var(-Hs_now.*((Hs_now./(repmat(HSB0,1,Nens).*exp(sig*HSB1))).^2-1));
        
        count_output=count_output+1;
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% PLOT
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if PLOT
        if mod(n,PLOT_INTERVAL)==1
            
            clf;
            fancyPlot;
            drawnow;
            
            if MAKE_ANIMATION % make gif animation
                
                % grab frame
                frame = getframe(gcf);
                im=frame2im(frame);
                
                [imind,cm]=rgb2ind(im,256);
                
                if count == 1;
                    imwrite(imind,cm,[filename '.gif'],'gif', 'Loopcount',inf,'delaytime',1/fps);
                else
                    imwrite(imind,cm,[filename '.gif'],'gif','WriteMode','append','delaytime',1/fps);
                end
                
                count=count+1;
                
            end
            
        end
    end
    
    if t(n+1)>=datenum(tstop,1,1)
        PLOT=0;
    end
    
    sim_time(mod(n,1000)+1)=toc;
    
    min_remaining=nanmean(sim_time)*(len-1-n)/60;
    
    if min_remaining>1
        fprintf('(~%0.1f min remaining)\n',min_remaining); % display progress
    else
        fprintf('(~%02d sec remaining)\n',round(min_remaining*60)); % display progress
    end

end

stop

toc2=toc(tic2);

% plot at the end
fancyPlot;

% print fig
print(gcf,[filename,'_Fig1_uniform_forcing.jpg'],'-djpeg','-r300');

figure;
plot(t_output,VAR,'-b.');

% save results
save('results_uniform_forcing.mat','t_output','VAR','DT','DY','HSB','dt','HSMAX','HSMEAN','HSVAR','FVAR');

DT_mean=nanmean(DT);
DY_mean=nanmean(DY);
HSB_mean=nanmean(HSB);

fprintf('DT (mean) = %.1f days\n',DT_mean);
fprintf('DY (mean) = %.1f m\n',DY_mean);
fprintf('Hs (mean) = %.1f m\n',HSB_mean);

b=HSB_mean.^2;
a=-HSB_mean.^2/DY_mean;
C=1/(DT_mean.*a.*HSB_mean);

fprintf('C (mean) = %.2f m^2/day, C (mean) = %.4f m^2/hr\n',C,C/24);
fprintf('a (mean) = %.2f m\n',a);
fprintf('b (mean) = %.2f m^2\n',b);
