% Comparing CoSMoS-COAST Xshore models w/ ensemble waves. SeanV.
clear all; close all; clc; 

% number of ensembles
Nens=1000;
    
% start time and time where ensemble forecast begins
tstart=datenum(1999,1,1);
tforecast1=datenum(2009,1,1);  % after this date, the ensemble forcing is used
tforecast2=datenum(2017,1,1);  % after this date, the ensemble forcing is used
tstop=datenum(2025,1,1);      % the simulation stops at this date

S1=load('results_uniform_forcing.mat');
%S2=load('results_ensemble_forcing.mat');
S2=load('results_ensemble_forcing_NEW_WAVES.mat');
%S2=load('results_ensemble_forcing_NEW_WAVES_highF.mat');

figure(1); set(gcf,'color','w','PaperPositionMode','auto','Position',[100 100 1000 600]); 

ax1 = axes; hold on; box on;

plot(S1.t_output,sqrt(S1.VAR),'b',S2.t_output,sqrt(S2.VAR),'r'); datetick('x');

VAR_MAX=max(S2.VAR)+8;

han1=plot([tforecast1 tforecast1],[0 sqrt(VAR_MAX)],'k--'); set(han1,'LineWidth',2);
han1=plot([tforecast2 tforecast2],[0 sqrt(VAR_MAX)],'k--'); set(han1,'LineWidth',2);

y_arrow=0.88;

annotation(gcf,'doublearrow',[0.378333333333334 0.4725],[y_arrow y_arrow]);

annotation(gcf,'doublearrow',[0.619500000000003 0.713666666666669],[y_arrow y_arrow]);

annotation(gcf,'textbox',[0.256833333333334 0.891666666666667 0.0681666666666665 0.0288888888888887],...
    'String',{'Hindcast','(Calibration)'},'FitBoxToText','off','LineStyle','none','FontSize',14,'HorizontalAlignment','center');

annotation(gcf,'textbox', [0.514500000000001 0.89 0.0681666666666666 0.0288888888888887],...
    'String',{'Forecast','(Validation)'},'FitBoxToText','off','LineStyle','none','FontSize',14,'HorizontalAlignment','center');

annotation(gcf,'textbox',[0.753500000000002 0.85 0.0864999999999979 0.0672222222222226],...
        'String',{'Projection'},'FitBoxToText','off','LineStyle','none','FontSize',14,'HorizontalAlignment','center');
    
set(gca,'Xlim',[tstart tstop]);
set(gca,'xtick',datenum(1999:2:2025,1,1)); datetick('x','keeplimits','keepticks'); axis tight;
set(gca,'Xlim',[tstart tstop]);
set(gca,'FontSize',14);

%han1=legend('uniform forcing','ensemble forcing'); set(han1,'position',[0.69366667054097 0.135555555555559 0.185999996125698 0.0833333311478303]);

xlabel('time','FontSize',20,'Interpreter','Latex')
ylabel('$\sigma_Y=\sqrt{\mathrm{var}(Y)}$ [m]','FontSize',20,'Interpreter','Latex')


% damping coefficients

% lambda1=nanmean(1-S1.dt./S1.DT.*S1.HSB./S1.DY);
% lambda2=nanmean(1-S2.dt./S2.DT.*S2.HSB./S2.DY);
% 
% lambda1_min=nanmin(1-S1.dt./S1.DT.*S1.HSB./S1.DY);
% lambda1_max=nanmax(1-S1.dt./S1.DT.*S1.HSB./S1.DY);
% 
% lambda2_min=nanmin(1-S2.dt./S2.DT.*S2.HSB./S2.DY);
% lambda2_max=nanmax(1-S2.dt./S2.DT.*S2.HSB./S2.DY);

R1=S1.HSMEAN./nanmean(S1.HSB);
fac1=nanmean(R1);

R2=S2.HSMEAN./nanmean(S2.HSB);
fac2=nanmean(R2);

sig_fac1=nanstd(R1);
sig_fac2=nanstd(R2);

lambda1=nanmedian(1-fac1*S1.dt./S1.DT);
lambda2=nanmedian(1-fac2*S2.dt./S2.DT);

lambda1_min=quantile(1-(fac1-sig_fac1)*S1.dt./S1.DT,0.1);
lambda1_max=quantile(1-(fac1+sig_fac1)*S1.dt./S1.DT,0.9);

lambda2_min=quantile(1-(fac2-sig_fac2)*S2.dt./S2.DT,0.1);
lambda2_max=quantile(1-(fac2+sig_fac2)*S2.dt./S2.DT,0.9);

fac=15;

sigmaf_full=nanmedian(sqrt(S2.FVAR));

sigmaf=nanmedian(sqrt(S2.FVAR(S2.t_output>tforecast2)));

sigma=0.2;
sigma_saw=0.17;

a1=1/lambda1.^2;   % the a coefficient based on lambda
a2=1/lambda2.^2;   % the a coefficient based on lambda

a1_min=1/lambda1_min.^2;   % the a coefficient based on lambda
a1_max=1/lambda1_max.^2;   % the a coefficient based on lambda

a2_min=1/lambda2_min.^2;   % the a coefficient based on lambda
a2_max=1/lambda2_max.^2;   % the a coefficient based on lambda

VAR_THEORY1=a1/(a1-1)*(sigma^2);                                       % the theoretical (yet approximate) upper bound for the model variance
VAR_THEORY2=a2/(a2-1)*(sigma^2+fac*(S2.dt/nanmedian(S2.DT)).^2*sigmaf^2);                                       % the theoretical (yet approximate) upper bound for the model variance

addative_noise=(sigma^2)
inherenent_noise= fac*(S2.dt/nanmedian(S2.DT)).^2*sigmaf^2

inherenent_noise/addative_noise

VAR_INC=VAR_THEORY2/VAR_THEORY1

stop

STD_INC=sqrt(VAR_THEORY2)/sqrt(VAR_THEORY1)

VAR_THEORY1_min=a1_min/(a1_min-1)*(sigma^2);                                       % the theoretical (yet approximate) upper bound for the model variance
VAR_THEORY1_max=a1_max/(a1_max-1)*(sigma^2);                                       % the theoretical (yet approximate) upper bound for the model variance

VAR_THEORY2_min=a2_min/(a2_min-1)*(sigma^2+fac*(S2.dt/nanmedian(S2.DT)).^2*sigmaf^2); % the theoretical (yet approximate) upper bound for the model variance
VAR_THEORY2_max=a2_max/(a2_max-1)*(sigma^2+fac*(S2.dt/nanmedian(S2.DT)).^2*sigmaf^2); % the theoretical (yet approximate) upper bound for the model variance

VAR_GROWTH_RATE1=(sigma^2)/S1.dt;
VAR_GROWTH_RATE2=(sigma_saw^2)/S1.dt;

han1=plot([tforecast1 tstop], [sqrt(VAR_THEORY1) sqrt(VAR_THEORY1)],'b--'); set(han1,'LineWidth',2);
han1=plot([tforecast2 tstop], [sqrt(VAR_THEORY2) sqrt(VAR_THEORY2)],'r--');       set(han1,'LineWidth',2);
% 
% plot([tforecast1 tstop], [VAR_THEORY1_min VAR_THEORY1_min],'b--');
% plot([tforecast1 tstop], [VAR_THEORY1_max VAR_THEORY1_max],'b--');
%  
% plot([tforecast2 tstop], [VAR_THEORY2_min VAR_THEORY2_min],'r--');
% plot([tforecast2 tstop], [VAR_THEORY2_max VAR_THEORY2_max],'r--');

BARS=0;
if BARS

han1=patch([tforecast1  tforecast2 tforecast2 tforecast2 tforecast1 tforecast1],sqrt([VAR_THEORY1_min VAR_THEORY1_min VAR_THEORY1 VAR_THEORY1_max VAR_THEORY1_max VAR_THEORY1]),[0 0 1 0 0 1]);
N=64; colormap(ax1,[linspace(1,0,N)' linspace(1,0,N)' linspace(1,1,N)']); 
set(han1,'EdgeColor','none','FaceAlpha',0.3);

% han1=patch([tforecast1  tstop tstop tstop tforecast1 tforecast1],sqrt([VAR_THEORY1_min VAR_THEORY1_min VAR_THEORY1 VAR_THEORY1_max VAR_THEORY1_max VAR_THEORY1]),[0 0 1 0 0 1]);
% N=64; colormap(ax1,[linspace(1,0,N)' linspace(1,0,N)' linspace(1,1,N)']); 
% set(han1,'EdgeColor','none','FaceAlpha',0.3);

ax2 = axes;

plot(S1.t_output,sqrt(S1.VAR),'b',S2.t_output,sqrt(S2.VAR),'r'); datetick('x');

han1=patch([tforecast2  tstop tstop tstop tforecast2 tforecast2],sqrt([VAR_THEORY2_min VAR_THEORY2_min VAR_THEORY2 VAR_THEORY2_max VAR_THEORY2_max VAR_THEORY2]),[0 0 1 0 0 1]);
N=64; colormap(ax2,[linspace(1,1,N)' linspace(1,0,N)' linspace(1,0,N)']); 
set(han1,'EdgeColor','none','FaceAlpha',0.3);

linkaxes([ax1,ax2])

ax2.Visible = 'off';
ax2.XTick = [];
ax2.YTick = [];


set(ax2,'children',flipud(get(ax2,'children')))

end

set(ax1,'children',flipud(get(ax1,'children')))

t1=7.3249e+05;
t2=7.3257e+05;
Y1=0.4;
Y2=2.25;

han1=rectangle('Position',[t1 Y1 t2-t1 Y2-Y1]); set(han1,'LineWidth',2);

annotation(gcf,'arrow',[0.326 0.304],[0.378333333333333 0.516666666666668]);

text(tstop+200,sqrt(VAR_THEORY1),'$\sigma_\mathrm{max}$','Interpreter','Latex','color','b','FontSize',20);
text(tstop+200,sqrt(VAR_THEORY2),'$\sigma_{F,\mathrm{max}}$','Interpreter','Latex','color','r','FontSize',20);

axes('Position',[0.174 0.525000000000004 0.238 0.26666666666667]); hold on; box on;

plot(S2.t_output,sqrt(S2.VAR),'r',S1.t_output,sqrt(S1.VAR),'b');
han1=plot((t1:t2)+17,sqrt(VAR_GROWTH_RATE1*[0:t2-t1]+0.3),'k--'); set(han1,'LineWidth',2);
han1=plot((t1:t2)+34,sqrt(VAR_GROWTH_RATE1*[0:t2-t1]+0.3),'k--'); set(han1,'LineWidth',2);
han1=plot((t1:t2)+50,sqrt(VAR_GROWTH_RATE1*[0:t2-t1]+0.3),'k--'); set(han1,'LineWidth',2);
%han1=plot((t1:t2)-2,mod(VAR_GROWTH_RATE2*[0:t2-t1],4.0)+0.3,'k--'); set(han1,'LineWidth',2);

set(gca,'Xlim',[t1 t2],'Ylim',[0.4 2.25],'xtick',t1:31:t2); 
datetick('x','keeplimits','keepticks'); axis tight;
set(gca,'Xlim',[t1 t2],'Ylim',[0.4 2.25],'xtick',t1:30:t2,'FontSize',12); 

han1=legend('ensemble forcing','uniform forcing'); 
set(han1,'position',[0.69366667054097 0.135555555555559 0.185999996125698 0.0833333311478303],'FontSize',14);


% print fig
print(gcf,'Fig3_ensemble_variance.jpg','-djpeg','-r300');

figure; id=S2.t_output>tforecast2; plot(S2.t_output(id),S2.VAR(id)-nanmean(S2.VAR(id)),'b',S2.t_output(id),S2.YMEAN(id),'r')

stop


%% more analysis

id=S1.t_output>tforecast1+100 & S1.t_output<tforecast2;  % flag the validation period

figure

subplot(2,1,1);
plot(S1.t_output,S1.HSMEAN,'-b.',S1.t_output(id),S1.HSMEAN(id),'-r.'); datetick('x');

subplot(2,1,2);
plot(S1.t_output,sqrt(S1.VAR),'-b.',S1.t_output(id),sqrt(S1.VAR(id)),'-r.'); datetick('x');

figure;
plot(S1.HSMEAN(id),sqrt(S1.VAR(id)),'b.')
