% variance and covariance of wave ensemble. SeanV.
close all; clc; %clear all;

Nens=1000; % the number of wave ensembles

RELOAD=1; % load in the ensemble wave data from Tairua
if RELOAD;
    
    %load('Results_100IndependentSimulations_3hourly.mat');
    load('Tairua_ensemble_waves_1000.mat');
    
    Nsim=size(Hs,2); % the number of wave ensembles
    
%     t=Simulation.time;  % time
%     Hs=Simulation.Hs;   % wave height
%     Tp=Simulation.Tm;   % wave period
%     Dir=Simulation.Dir; % wave direction
    
    if  Nens~=Nsim; % if the number of model ensembles is not the same as the number of wave ensebles  
        Hs=Hs(:,randi([1 Nsim],Nens,1));
        Tp=Tp(:,randi([1 Nsim],Nens,1));
        Dir=Dir(:,randi([1 Nsim],Nens,1));     
    end
    
end

len=length(t); % length of the wave time series

dt=nanmean(diff(t));  % model time step

Hs_bar=nanmean(Hs(:));            % mean wave height

DT=28;   % model time scale
DY=10;   % model shoreline excursion parameter

r1=-0.05; % rate
r2=-2; % rate

Nsteps=10000;  % number of steps to run the model for

idst=1000;     % start id

t0=t(idst);            % the inital time
t_mod=t(idst:idst+Nsteps-1,:)-t0; % truncate the time series to only run the model for Nsteps
f_mod=f(idst:idst+Nsteps-1,:);    % truncate the forcing time series to only run the model for Nsteps

ti=t_mod(1);       % initial time for plotting
tstop=t_mod(end);  % final time for plotting

MIN=-180;
MAX=60;            % upper limit for model state for plotting

% RUN FORWARD MODEL
Y=NaN(Nsteps,Nens);
Y(1,:)=0;           

Y1=NaN(Nsteps,Nens); 
Y1(1,:)=0;           

Y2=NaN(Nsteps,Nens); 
Y2(1,:)=0;           

Y3=NaN(Nsteps,Nens); 
Y3(1,:)=0;           

for n=1:Nsteps-1

    t=t_mod(n)-ti;
    
    % forward model, real forcing
    Yeq=-DY*(Hs(n,:).^2-Hs_bar^2)./Hs_bar^2;
    tau=DT*(Hs_bar./Hs(n,:));
    
    % Yates (unmodified)
    Y(n+1,:)=Y(n,:)+dt./tau.*(Yeq-Y(n,:));
    
    % Yates with actual trend
    Y1(n+1,:)=Y1(n,:)+dt./tau.*((Yeq+r1*t)-Y1(n,:));
    
    % Yates with RHS "trend"
    Y2(n+1,:)=Y2(n,:)+dt./tau.*(Yeq-Y2(n,:))+dt*r2;

    % Yates with RHS "trend"
    Y3(n+1,:)=Y3(n,:)+dt./tau.*((Yeq+r1*t)-Y3(n,:))+dt*r2;
end

PRINT=1;

x0=0.1;
y0=0.08;
width=0.80;
height=0.43;
ysep=0.04;

%% Generate fig

figure(1);
set(gcf,'Color','w','PaperPositionMode','auto','Position',[100 100 1000 800]);

subplot('position',[x0 y0+height+ysep width height]);  hold on; box on;

sig_min=-50; % min Y at the end of the simulation
sig_max=10; % max Y at the end of the simulation
cmap=flipud(jet(128)); % color map

for i=1:Nens
    sig=Y(end,i); % Y at the end of the simulation
    
    c=round(interp1([sig_min sig_max],[1 128],sig)); % figure out the color of a specific line
    
    if sig>sig_max
        c=128;
    elseif sig<sig_min
        c=1;
    end
    
    patchline(t_mod,Y(:,i),'linewidth',0.5,'edgealpha',0.4,'edgecolor',cmap(c,:)); % make a patchline for each ensemble member
end

han1=plot([min(t_mod) max(t_mod)],[0 0],'k--'); set(han1,'LineWidth',2);

set(gca,'FontSize',16,'Ytick',MIN:20:MAX);
%xlabel('time','interpreter','latex'); 
ylabel('$Y [m]$','interpreter','latex','FontSize',20);
axis([ti tstop MIN MAX]);

set(gca,'Xticklabel',[],'layer','top')

subplot('position',[x0 y0 width height]); hold on; box on;
sig_min=r1*(tstop-ti)-50; % min Y at the end of the simulation
sig_max=r1*(tstop-ti)+10; % max Y at the end of the simulation
cmap=flipud(jet(128)); % color map

for i=1:Nens
    sig=Y1(end,i); % Y at the end of the simulation
    
    c=round(interp1([sig_min sig_max],[1 128],sig)); % figure out the color of a specific line
    
    if sig>sig_max
        c=128;
    elseif sig<sig_min
        c=1;
    end
    
    patchline(t_mod,Y1(:,i),'linewidth',0.5,'edgealpha',0.4,'edgecolor',cmap(c,:)); % make a patchline for each ensemble member
end

han1=plot([min(t_mod) max(t_mod)],[0 0],'k--'); set(han1,'LineWidth',2);
han1=plot([min(t_mod) max(t_mod)],[0 r1*(tstop-ti)],'m--'); set(han1,'LineWidth',2);

set(gca,'FontSize',16,'Ytick',-240:20:80);
xlabel('time [days]','interpreter','latex'); 
ylabel('$Y [m]$','interpreter','latex','FontSize',20);
axis([ti tstop MIN+r1*(tstop-ti)  MAX]);

set(gca,'layer','top')

annotation(gcf,'textbox',[0.159 0.8775 0.128 0.0712499999999998],'String',{'$\frac{dY}{dt}=\frac{1}{\tau}(Y_{eq}-Y)$'},'FitBoxToText','off','interpreter','latex','FontSize',18,'linestyle','none');
annotation(gcf,'textbox',[0.159 0.8775-height-ysep 0.128 0.0712499999999998],'String',{'$\frac{dY}{dt}=\frac{1}{\tau}([Y_{eq}+r_1t]-Y)$'},'FitBoxToText','off','interpreter','latex','FontSize',18,'linestyle','none');
annotation(gcf,'textbox',[0.793000000000001 0.365 0.052 0.0487499999999998],'String',{'$r_1t$'},'FitBoxToText','off','interpreter','latex','FontSize',18,'linestyle','none');

annotation(gcf,'doublearrow',[0.793 0.792],[0.35125 0.42125]);

if PRINT
    print(gcf,'Yates_test.jpg','-djpeg','-r300');
end

%%
figure(2);
set(gcf,'Color','w','PaperPositionMode','auto','Position',[100 100 1000 800]);

subplot('position',[x0 y0+height+ysep width height]);  hold on; box on;

sig_min=-50; % min Y at the end of the simulation
sig_max=10; % max Y at the end of the simulation
cmap=flipud(jet(128)); % color map

for i=1:Nens
    sig=Y(end,i); % Y at the end of the simulation
    
    c=round(interp1([sig_min sig_max],[1 128],sig)); % figure out the color of a specific line
    
    if sig>sig_max
        c=128;
    elseif sig<sig_min
        c=1;
    end
    
    patchline(t_mod,Y(:,i),'linewidth',0.5,'edgealpha',0.4,'edgecolor',cmap(c,:)); % make a patchline for each ensemble member
end

han1=plot([min(t_mod) max(t_mod)],[0 0],'k--'); set(han1,'LineWidth',2);

set(gca,'FontSize',16,'Ytick',MIN:20:MAX);
%xlabel('time','interpreter','latex'); 
ylabel('$Y [m]$','interpreter','latex','FontSize',20);
axis([ti tstop MIN MAX]);

set(gca,'Xticklabel',[],'layer','top')

subplot('position',[x0 y0 width height]); hold on; box on;
sig_min=r2*DT-50; % min Y at the end of the simulation
sig_max=r2*DT+10; % max Y at the end of the simulation
cmap=flipud(jet(128)); % color map

for i=1:Nens
    sig=Y2(end,i); % Y at the end of the simulation
    
    c=round(interp1([sig_min sig_max],[1 128],sig)); % figure out the color of a specific line
    
    if sig>sig_max
        c=128;
    elseif sig<sig_min
        c=1;
    end
    
    patchline(t_mod,Y2(:,i),'linewidth',0.5,'edgealpha',0.4,'edgecolor',cmap(c,:)); % make a patchline for each ensemble member
end

han1=plot(t_mod,r2*DT*(1-exp(-t_mod/DT)),'r--'); set(han1,'LineWidth',3);

han1=plot([min(t_mod) max(t_mod)],[0 0],'k--'); set(han1,'LineWidth',2);
han1=plot([min(t_mod) max(t_mod)],[r2*DT r2*DT],'m--'); set(han1,'LineWidth',2);


set(gca,'FontSize',16,'Ytick',-240:20:80);
xlabel('time [days]','interpreter','latex'); 
ylabel('$Y [m]$','interpreter','latex','FontSize',20);
axis([ti tstop MIN MAX ]);

set(gca,'layer','top')

annotation(gcf,'textbox',[0.159 0.8775 0.128 0.0712499999999998],'String',{'$\frac{dY}{dt}=\frac{1}{\tau}(Y_{eq}-Y)$'},'FitBoxToText','off','interpreter','latex','FontSize',18,'linestyle','none');
annotation(gcf,'textbox',[0.159 0.8775-height-ysep 0.128 0.0712499999999998],'String',{'$\frac{dY}{dt}=\frac{1}{\tau}(Y_{eq}-Y)+r_2$'},'FitBoxToText','off','interpreter','latex','FontSize',18,'linestyle','none');
annotation(gcf,'textbox',[0.584000000000001 0.33 0.052 0.0487499999999998],'String',{'$r_2\tau$'},'FitBoxToText','off','interpreter','latex','FontSize',18,'linestyle','none');

annotation(gcf,'textbox',[0.2 0.33 0.2 0.0487499999999998],'String',{'$r_2\tau(1-\exp(\frac{t}{\tau}))$'},'FitBoxToText','off','interpreter','latex','FontSize',18,'linestyle','none');

annotation(gcf,'textarrow',[0.201 0.115],[0.3565 0.35625]);
annotation(gcf,'doublearrow',[0.585 0.584],[0.305 0.39875]);

if PRINT
    print(gcf,'Yates_test2.jpg','-djpeg','-r300');
end

%%
figure(3);
set(gcf,'Color','w','PaperPositionMode','auto','Position',[100 100 1000 800]);

subplot('position',[x0 y0+height+ysep width height]);  hold on; box on;

sig_min=-50; % min Y at the end of the simulation
sig_max=10; % max Y at the end of the simulation
cmap=flipud(jet(128)); % color map

for i=1:Nens
    sig=Y(end,i); % Y at the end of the simulation
    
    c=round(interp1([sig_min sig_max],[1 128],sig)); % figure out the color of a specific line
    
    if sig>sig_max
        c=128;
    elseif sig<sig_min
        c=1;
    end
    
    patchline(t_mod,Y(:,i),'linewidth',0.5,'edgealpha',0.4,'edgecolor',cmap(c,:)); % make a patchline for each ensemble member
end

han1=plot([min(t_mod) max(t_mod)],[0 0],'k--'); set(han1,'LineWidth',2);

set(gca,'FontSize',16,'Ytick',MIN:20:MAX);
%xlabel('time','interpreter','latex'); 
ylabel('$Y [m]$','interpreter','latex','FontSize',20);
axis([ti tstop MIN MAX]);

set(gca,'Xticklabel',[],'layer','top')

subplot('position',[x0 y0 width height]); hold on; box on;
sig_min=r1*(tstop-ti)+r2*DT-50; % min Y at the end of the simulation
sig_max=r1*(tstop-ti)+r2*DT+10; % max Y at the end of the simulation
cmap=flipud(jet(128)); % color map

for i=1:Nens
    sig=Y3(end,i); % Y at the end of the simulation
    
    c=round(interp1([sig_min sig_max],[1 128],sig)); % figure out the color of a specific line
    
    if sig>sig_max
        c=128;
    elseif sig<sig_min
        c=1;
    end
    
    patchline(t_mod,Y3(:,i),'linewidth',0.5,'edgealpha',0.4,'edgecolor',cmap(c,:)); % make a patchline for each ensemble member
end

han1=plot(t_mod,r2*DT*(1-exp(-t_mod/DT)),'m--'); set(han1,'LineWidth',3);

han1=plot(t_mod,r1*(t_mod-ti)+r2*DT*(1-exp(-t_mod/DT)),'r--'); set(han1,'LineWidth',3);

han1=plot([min(t_mod) max(t_mod)],[0 0],'k--'); set(han1,'LineWidth',2);

set(gca,'FontSize',16,'Ytick',-240:20:80);
xlabel('time [days]','interpreter','latex'); 
ylabel('$Y [m]$','interpreter','latex','FontSize',20);
axis([ti tstop MIN MAX ]);

set(gca,'layer','top')

annotation(gcf,'textbox',[0.159 0.8775 0.128 0.0712499999999998],'String',{'$\frac{dY}{dt}=\frac{1}{\tau}(Y_{eq}-Y)$'},'FitBoxToText','off','interpreter','latex','FontSize',18,'linestyle','none');
annotation(gcf,'textbox',[0.159 0.8775-height-ysep 0.128 0.0712499999999998],'String',{'$\frac{dY}{dt}=\frac{1}{\tau}([Y_{eq}+r_1t]-Y)+r_2$'},'FitBoxToText','off','interpreter','latex','FontSize',18,'linestyle','none');
annotation(gcf,'textbox',[0.584000000000001 0.33 0.052 0.0487499999999998],'String',{'$r_2\tau$'},'FitBoxToText','off','interpreter','latex','FontSize',18,'linestyle','none');
annotation(gcf,'textbox',[0.584000000000001 0.245 0.052 0.0487499999999998],'String',{'$r_1t$'},'FitBoxToText','off','interpreter','latex','FontSize',18,'linestyle','none');

annotation(gcf,'textbox',[0.2 0.33 0.2 0.0487499999999998],'String',{'$r_1t+r_2\tau(1-\exp(\frac{t}{\tau}))$'},'FitBoxToText','off','interpreter','latex','FontSize',18,'linestyle','none');

annotation(gcf,'textarrow',[0.201 0.115],[0.3565 0.35625]);
annotation(gcf,'doublearrow',[0.585 0.584],[0.305 0.39875]);
annotation(gcf,'doublearrow',[0.585 0.585],[0.23375 0.30375]);

if PRINT
    print(gcf,'Yates_test3.jpg','-djpeg','-r300');
end

