% Loading in the Laura's waves from Tairua. SeanV.
clear all; close all; clc;

% start time and time where ensemble forecast begins
tstart=datenum(1999,1,1);      % the simulation startes on this date
tforecast1=datenum(2009,1,1);  % after this date, the ensemble forcing is used
tforecast2=datenum(2017,1,1);  % after this date, the ensemble forcing is used
tstop=datenum(2025,1,1);       % the simulation stops at this date

dt=3/24;              % 3 hour time step
t=(tstart:dt:tstop)'; % the simulation time
len=length(t);

sim_time=NaN(100,1);

RELOAD=0;
if RELOAD
    
    % waves data location
    WAVE_DIR='Simulation_KernelHs_KernelTm_1000Ind_DailyProb';
    
    % get a list of the waves files
    drr=dir([WAVE_DIR,filesep,'*.mat']);
    files={drr.name}';
    id=strcmp(files,'Dates_time.mat');
    
    % load the times vector
    load([WAVE_DIR,filesep,files{id}]); files(id)=[];
    t1=datenum(dates(:,1),dates(:,2),dates(:,3),dates(:,4),dates(:,5),dates(:,6));
    
    % find where the waves simulation times overlap with the shoreline
    % simulation times
    id1=find(t1>=tstart,1,'first');
    id2=find(t1>=tstop,1,'first');
    
    % number of wave ensembles
    Nens=length(files);
    
    % pre-allocate
    Hs=NaN(len,Nens);
    Tp=NaN(len,Nens);
    Dir=NaN(len,Nens);
    
    % load waves data one at a time
    for n=1:Nens
        
        tic % start timer
        
        fprintf('working on file %d of %d ... ',n,Nens); % display progress
        
        load([WAVE_DIR,filesep,files{n}]); % load one waves file
        
        Hs(:,n)=HS(id1:id2); % get data
        Tp(:,n)=TM(id1:id2);
        Dir(:,n)=DIR(id1:id2);
        
        fprintf('done. ');
        
        % keep track of time remaining
        sim_time(mod(n,100)+1)=toc;
        
        min_remaining=nanmean(sim_time)*(Nens-1-n)/60;
        
        if min_remaining>1
            fprintf('(~%0.1f min remaining)\n',min_remaining); % display progress
        else
            fprintf('(~%02d sec remaining)\n',round(min_remaining*60)); % display progress
        end
        
    end
    
    save('Tairua_ensemble_waves_1000.mat','t','Hs','Tp','Dir'); % save results
    
else
    load('Tairua_ensemble_waves_1000.mat'); % load results
end

Nens=size(Hs,2);

%plot(t,Hs); datetick('x','keeplimits','keepticks');

RECOMPUTE_STATS=0;

if RECOMPUTE_STATS

    NENS=[2 5 10 20 30 40 50 60 70 80 90 100 125 150 200 250 300 400 500 600 700 800 900 1000]';
    
    Ntrials=100;   % Number of traials

    N=5000;        % based on first N elements in wave time series
    
    Nmore=8*10;    % where to truncate covariance matrix (8 records per day x 10 days)
    
    % preallocate
    VAR=NaN(length(NENS),Ntrials);
    FAC=NaN(length(NENS),Ntrials);
    SIGF=NaN(length(NENS),Ntrials);
       
    %IDS=NaN(length(NENS_small),Ntrials,Ntrials);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % find the factor, fac
    Hs_bar=nanmean(Hs(:));            % mean wave height
    DY=10;
    f=-DY.*Hs./Hs_bar.*(Hs.^2-Hs_bar^2)./Hs_bar^2;  % forcing
    
    for i=1:length(NENS)
        for j=1:Ntrials
            
            fprintf('i=%d (of %d),j=%d (of %d) ...\n',i,length(NENS),j,Ntrials);

            covt=zeros(N,N); % covariance matrix
            
            id=randi(Nens,NENS(i),1);
            
            %IDS(i,j,:)=id;
            
            VAR(i,j)=var(reshape(Hs(:,id),[],1));
            
            for ii=1:N      % for all N times
                
                jmin=max(1,ii-Nmore);
                jmax=min(N,ii+Nmore);
                
                for jj=jmin:jmax
                    
                    A=cov(f(ii,id),f(jj,id)); % calculate ...
                    covt(ii,jj)=A(2,1);       % and save covariance matrix
                    covt(ii,jj)=A(2,1);       % and save covariance matrix
                    
                end
            end
            
            %         figure;
            %         pcolor(covt); shading flat; colormap(jet); colorbar; axis ij; % plot covaraince matrix
            %         stop
            
            Svar=sum(diag(covt));     % get trace of covariance matrix
            A=covt-diag(diag(covt));  % remove diagonal elements from covaraince matrix
            Scov=sum(A(:));           % get sum of the off diagonal elements of the covaraince matrix
            
            R=Scov/Svar; % the ratio between the total covariance part and the variance part, which gives a scaling factor
            
            fac=1+R;
            
            % save results
            FAC(i,j)=fac;
            SIGF(i,j)=std(reshape(f(:,id),[],1));
            
        end
    end
    
    save('Tairua_ensemble_wave_stats_1000.mat','VAR','FAC','NENS','SIGF','N','Ntrials','Nmore'); % save results
    
else
    load('Tairua_ensemble_wave_stats_1000.mat');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% plot

figure; hold on ; box on;
ALPHA=0.2;

CI1=quantile(VAR,0.025,2);
CI2=quantile(VAR,0.05,2);
CI3=quantile(VAR,0.95,2);
CI4=quantile(VAR,0.975,2);

han1=fill([NENS; flipud(NENS)],[CI1; flipud(CI2)],'y');  set(han1,'FaceAlpha',ALPHA,'EdgeColor','none');
han1=fill([NENS; flipud(NENS)],[CI2; flipud(CI3)],'g');  set(han1,'FaceAlpha',ALPHA,'EdgeColor','none');
han1=fill([NENS; flipud(NENS)],[CI3; flipud(CI4)],'y');  set(han1,'FaceAlpha',ALPHA,'EdgeColor','none');

plot(NENS,nanmedian(VAR,2),'-r.')
xlabel('# of ensembles');    
ylabel('variance of H_s');  

figure; hold on ; box on;
ALPHA=0.2;

CI1=quantile(FAC,0.025,2);
CI2=quantile(FAC,0.05,2);
CI3=quantile(FAC,0.95,2);
CI4=quantile(FAC,0.975,2);

han1=fill([NENS; flipud(NENS)],[CI1; flipud(CI2)],'y');  set(han1,'FaceAlpha',ALPHA,'EdgeColor','none');
han1=fill([NENS; flipud(NENS)],[CI2; flipud(CI3)],'g');  set(han1,'FaceAlpha',ALPHA,'EdgeColor','none');
han1=fill([NENS; flipud(NENS)],[CI3; flipud(CI4)],'y');  set(han1,'FaceAlpha',ALPHA,'EdgeColor','none');

plot(NENS,CI4,'-k.')
plot(NENS,CI1,'-k.')

plot(NENS,nanmedian(FAC,2),'-r.')
xlabel('# of ensembles');    
ylabel('factor F');  

figure; hold on ; box on;
ALPHA=0.2;

CI1=quantile(SIGF,0.025,2);
CI2=quantile(SIGF,0.05,2);
CI3=quantile(SIGF,0.95,2);
CI4=quantile(SIGF,0.975,2);

han1=fill([NENS; flipud(NENS)],[CI1; flipud(CI2)],'y');  set(han1,'FaceAlpha',ALPHA,'EdgeColor','none');
han1=fill([NENS; flipud(NENS)],[CI2; flipud(CI3)],'g');  set(han1,'FaceAlpha',ALPHA,'EdgeColor','none');
han1=fill([NENS; flipud(NENS)],[CI3; flipud(CI4)],'y');  set(han1,'FaceAlpha',ALPHA,'EdgeColor','none');

plot(NENS,nanmedian(SIGF,2),'-r.')
xlabel('# of ensembles');    
ylabel('std of f');   

stop

if 0
    
    NENS_small=5;
    
    Ntrials=20000;
    VAR_small=NaN(length(NENS_small),Ntrials);
    IDS=NaN(length(NENS_small),Ntrials,NENS_small);
    
    for i=1:length(NENS_small)
        for j=1:Ntrials
            fprintf('i=%d (of %d),j=%d (of %d) ...\n',i,length(NENS_small),j,Ntrials);
            id=randi(Nens,NENS_small(i),1);
            IDS(i,j,:)=id;
            VAR_small(i,j)=var(reshape(Hs(:,id),[],1));
        end
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % plot
    
    figure; hold on ; box on;
    [~,id]=sort(VAR_small,'ascend');
    VAR_small=VAR_small(1,id);
    IDS=IDS(:,id,:);
    IDS=sort(IDS,3);
    
    VAR_small=squeeze(VAR_small);
    IDS=squeeze(IDS);
    
    Hs_bar=nanmean(Hs(:));            % mean wave height
    DY=10;
    
    f1=-DY.*Hs(:,IDS(1  ,:))./Hs_bar.*(Hs(:,IDS(1  ,:)).^2-Hs_bar^2)./Hs_bar^2;  % forcing
    f2=-DY.*Hs(:,IDS(end,:))./Hs_bar.*(Hs(:,IDS(end,:)).^2-Hs_bar^2)./Hs_bar^2;  % forcing
    
    N=5000;        % based on first N elements in wave time series
    
    covt1=zeros(N,N); % covariance matrix
    covt2=zeros(N,N); % covariance matrix
    
    for ii=1:N      % for all N times
        
        jmin=max(1,ii-Nmore);
        jmax=min(N,ii+Nmore);
        
        for jj=jmin:jmax
            
            A=cov(f1(ii,:),f1(jj,:)); % calculate ...
            covt1(ii,jj)=A(2,1);       % and save covariance matrix
            covt1(ii,jj)=A(2,1);       % and save covariance matrix
            
            A=cov(f2(ii,:),f2(jj,:)); % calculate ...
            covt2(ii,jj)=A(2,1);       % and save covariance matrix
            covt2(ii,jj)=A(2,1);       % and save covariance matrix
            
        end
    end
    
    %         figure;
    %         pcolor(covt); shading flat; colormap(jet); colorbar; axis ij; % plot covaraince matrix
    %         stop
    
    Svar1=sum(diag(covt1));     % get trace of covariance matrix
    A=covt1-diag(diag(covt1));  % remove diagonal elements from covaraince matrix
    Scov1=sum(A(:));           % get sum of the off diagonal elements of the covaraince matrix
    
    Svar2=sum(diag(covt2));     % get trace of covariance matrix
    A=covt2-diag(diag(covt2));  % remove diagonal elements from covaraince matrix
    Scov2=sum(A(:));           % get sum of the off diagonal elements of the covaraince matrix
    
    R1=Scov1/Svar1; % the ratio between the total covariance part and the variance part, which gives a scaling factor
    R2=Scov2/Svar2; % the ratio between the total covariance part and the variance part, which gives a scaling factor
    
    % save results
    F1=1+R1;
    F2=1+R2;

    SIGF1=std(reshape(f1,[],1));
    SIGF2=std(reshape(f2,[],1));
    
    figure;
    plot(1:Ntrials,VAR_small,'-b.');

    figure;
    
    subplot(2,1,1)
    plot(t,Hs(:,IDS(1,:))); axis([tstart tstop 0 5]); datetick('x','keeplimits','keepticks');
    
    subplot(2,1,2);
    plot(t,Hs(:,IDS(end,:))); axis([tstart tstop 0 5]); datetick('x','keeplimits','keepticks');
    
    stop
    
end

if 0
    
    NENS_small=5;

    Ntrials=500;   % Number of traials

    N=10000;        % based on first N elements in wave time series
    
    Nmore=8*10;    % where to truncate covariance matrix (8 records per day x 10 days)
    
    % preallocate  
    VAR=NaN(length(NENS_small),Ntrials);
    FAC=NaN(length(NENS_small),Ntrials);
    SIGF=NaN(length(NENS_small),Ntrials);
    
    IDS=NaN(length(NENS_small),Ntrials,NENS_small);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % find the factor, fac
    Hs_bar=nanmean(Hs(:));            % mean wave height
    DY=10;
    
    for i=1:length(NENS_small)
        for j=1:Ntrials
            
            fprintf('i=%d (of %d),j=%d (of %d) ...\n',i,length(NENS_small),j,Ntrials);

            covt=zeros(N,N); % covariance matrix
            
            id=randi(Nens,NENS_small(i),1);
            
            IDS(i,j,:)=id;
            
            VAR(i,j)=var(reshape(Hs(:,id),[],1));
            
            f=-DY.*Hs(t>tforecast2,id)./Hs_bar.*(Hs(t>tforecast2,id).^2-Hs_bar^2)./Hs_bar^2;  % forcing
            
            for ii=1:N      % for all N times
                
                jmin=max(1,ii-Nmore);
                jmax=min(N,ii+Nmore);
                
                for jj=jmin:jmax
                    
                    A=cov(f(ii,:),f(jj,:)); % calculate ...
                    covt(ii,jj)=A(2,1);       % and save covariance matrix
                    covt(ii,jj)=A(2,1);       % and save covariance matrix
                    
                end
            end
            
            %         figure;
            %         pcolor(covt); shading flat; colormap(jet); colorbar; axis ij; % plot covaraince matrix
            %         stop
            
            Svar=sum(diag(covt));     % get trace of covariance matrix
            A=covt-diag(diag(covt));  % remove diagonal elements from covaraince matrix
            Scov=sum(A(:));           % get sum of the off diagonal elements of the covaraince matrix
            
            R=Scov/Svar; % the ratio between the total covariance part and the variance part, which gives a scaling factor
            
            fac=1+R;
            
            % save results
            FAC(i,j)=fac;
            SIGF(i,j)=std(reshape(f,[],1));
            
        end
    end

    VAR=squeeze(VAR);
    FAC=squeeze(FAC);
    SIGF=squeeze(SIGF);
    IDS=squeeze(IDS);
    
    [~,id]=sort(FAC,'ascend');
    
    FAC=FAC(id);
    VAR=VAR(id);
    SIGF=SIGF(id);
    IDS=IDS(id,:);
    
    IDS=sort(IDS,2);
        
    save('Tairua_ensemble_wave_stats_1000_Nens5.mat','VAR','FAC','SIGF','IDS','NENS_small','N','Ntrials','Nmore'); % save results

else
    load('Tairua_ensemble_wave_stats_1000_Nens5.mat');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% plot

%low F: 118   139   349   617   714
%high F: 316   361   560   699   973

figure;
plot(1:Ntrials,FAC,'-b.');

figure;

subplot(2,1,1)
plot(t,Hs(:,IDS(1,:))); axis([tstart tstop 0 5]); datetick('x','keeplimits','keepticks');

subplot(2,1,2);
plot(t,Hs(:,IDS(end,:))); axis([tstart tstop 0 5]); datetick('x','keeplimits','keepticks');

nanmedian(FAC)

IDS(1,:)
IDS(end,:)

F1=FAC(1)
F2=FAC(end)

SIGF1=SIGF(1)
SIGF2=SIGF(end)

