% CoSMoS-COAST Xshore model w/ ensemble waves. SeanV.
close all; clc; %clear all;

% seed random number generator
seed=1;
rng(seed);

% number of ensembles
Nens=10000;

% time stepping
t0=0;
dt=0.1;
tstop=400;

t=(t0:dt:tstop)';

len=length(t);

Y=NaN(len,Nens);  Y (1,:)=0; % initial conditions
Y2=NaN(len,Nens); Y2(1,:)=0; 
Y3=NaN(len,Nens); Y3(1,:)=0; 
Y4=NaN(len,Nens); Y4(1,:)=0; 
Y5=NaN(len,Nens); Y5(1,:)=0; 

Var =NaN(len,1); Var(1) =var( Y(1,:));
Var2=NaN(len,1); Var2(1)=var(Y2(1,:));
Var3=NaN(len,1); Var3(1)=var(Y3(1,:));
Var4=NaN(len,1); Var4(1)=var(Y4(1,:));
Var5=NaN(len,1); Var5(1)=var(Y5(1,:));

sigma=0.1;

sigmaf=0.3;

DT=200*dt;

DT2=2*dt;

VAR_GROWTH_RATE=sigma^2/dt;

VAR_GROWTH_RATE2=(sigma^2+(dt/DT2)^2*sigmaf^2)/dt;

NOISE=sigma/dt;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% MAIN Simulation (with data assimilation from previous run)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% main time stepping loop
for n=1:len-1;

    % random walk
    Y(n+1,:)  =Y(n,:)+dt*NOISE*randn(1,Nens);
    
    % random walk + damping
    Y2(n+1,:) =Y2(n,:)+dt*NOISE*randn(1,Nens)-dt/DT*Y2(n,:);
    
    % random walk + damping + forcing
    Y3(n+1,:) =Y3(n,:)+dt*NOISE*randn(1,Nens)-dt/DT*Y3(n,:)+dt/DT2*(sigmaf*randn(1,Nens));

    % random walk + damping + (uniform) forcing
    Y4(n+1,:) =Y4(n,:)+dt*NOISE*randn(1,Nens)-dt/DT*Y4(n,:)+dt/DT2*(sigmaf*repmat(randn(1),1,Nens));
    
    % random walk + damping + (uniform) forcing for the first half of the run, then ensemble forcing for the second half
    if t(n+1)<tstop/2
        Y5(n+1,:) =Y5(n,:)+dt*NOISE*randn(1,Nens)-dt/DT*Y5(n,:)+dt/DT2*(sigmaf*repmat(randn(1),1,Nens));    % uniform forcing
    else
        Y5(n+1,:) =Y5(n,:)+dt*NOISE*randn(1,Nens)-dt/DT*Y5(n,:)+dt/DT2*(sigmaf*randn(1,Nens));              % ensemble forcing
    end
    
    Var(n+1) =var( Y(n+1,:));
    Var2(n+1)=var(Y2(n+1,:));
    Var3(n+1)=var(Y3(n+1,:));
    Var4(n+1)=var(Y4(n+1,:));
    Var5(n+1)=var(Y5(n+1,:));
    
end

a=1/(1-dt/DT).^2;

VAR_THEORY=a/(a-1)*sigma^2;
VAR_THEORY2=a/(a-1)*(sigma^2+(dt/DT2).^2*sigmaf^2);

UPPER=sqrt(VAR_THEORY);
UPPER2=sqrt(VAR_THEORY2);

MAX_fig1=25;
MAX=10;
MAX2=6;
MAX2_fig1=41;
interval=14;

PRINT=1;

x0=0.1;
y0=0.08;
width=0.85;
height=0.43;
ysep=0.04;

figure(1);
set(gcf,'Color','w','PaperPositionMode','auto','Position',[100 100 1000 800]);

subplot('position',[x0 y0+height+ysep width height]);  hold on; box on;

sig_min=min(Y(end,:)); % min Y at the end of the simulation
sig_max=max(Y(end,:)); % max Y at the end of the simulation
cmap=flipud(jet(128)); % color map

for i=1:Nens
    sig=Y(end,i); % Y at the end of the simulation
    c=round(interp1([sig_min sig_max],[1 128],sig)); % figure out the color of a specific line
    patchline(t,Y(:,i),'linewidth',0.5,'edgealpha',0.4,'edgecolor',cmap(c,:)); % make a patchline for each ensemble member
end

han1=plot(t,0*t+3*UPPER,'k--',t,0*t-3*UPPER,'k--'); set(han1,'LineWidth',2);
set(gca,'FontSize',16,'Ytick',-MAX_fig1:5:MAX_fig1);
%xlabel('time','interpreter','latex'); 
ylabel('$Y$','interpreter','latex','FontSize',20);
axis([t0 tstop -MAX_fig1 MAX_fig1]);

han1=text(5,3*UPPER+2,'$+3\sigma$'); set(han1,'interpreter','latex','FontSize',18);
han1=text(5,-3*UPPER-2,'$-3\sigma$'); set(han1,'interpreter','latex','FontSize',18);

set(gca,'Xticklabel',[],'layer','top')

subplot('position',[x0 y0 width height]); hold on; box on;
han1=plot(t,Var,'k'); set(han1,'LineWidth',1);
han1=plot(t,VAR_GROWTH_RATE*t,'r--'); set(han1,'LineWidth',2);

han1=legend('var(Y)','$\sigma^2 \frac{t}{\Delta t}$'); set(han1,'location','northwest','interpreter','latex','FontSize',20);
set(gca,'FontSize',16,'layer','top');
xlabel('time','interpreter','latex','FontSize',20); ylabel('var($Y$)','interpreter','latex','FontSize',20); 
axis([t0 tstop 0 MAX2_fig1]);

annotation(gcf,'textbox',[0.117 0.925 0.199 0.0475],'String',{'random walk'},'FitBoxToText','off','linestyle','none','FontSize',18);

if PRINT
    print(gcf,'Fig_6_variance_growth.jpg','-djpeg','-r300');
end

figure(2);
set(gcf,'Color','w','PaperPositionMode','auto','Position',[100 100 1000 800]);

subplot('position',[x0 y0+height+ysep width height]);  hold on; box on;

sig_min=min(Y2(end,:)); % min Y at the end of the simulation
sig_max=max(Y2(end,:)); % max Y at the end of the simulation
cmap=flipud(jet(128)); % color map

for i=1:Nens
    sig=Y2(end,i); % Y at the end of the simulation
    c=round(interp1([sig_min sig_max],[1 128],sig)); % figure out the color of a specific line
    patchline(t,Y2(:,i),'linewidth',0.5,'edgealpha',0.4,'edgecolor',cmap(c,:)); % make a patchline for each ensemble member
end

han1=plot(t,0*t+3*UPPER,'b--',t,0*t-3*UPPER,'b--'); set(han1,'LineWidth',2);
set(gca,'FontSize',16,'Ytick',-MAX:2:MAX);
ylabel('$Y$','interpreter','latex','FontSize',20);
axis([t0 tstop -MAX MAX]);

han1=text(1,3*UPPER+1,'$+3\sigma_\mathrm{max}$'); set(han1,'interpreter','latex','FontSize',18);
han1=text(1,-3*UPPER-1.15,'$-3\sigma_\mathrm{max}$'); set(han1,'interpreter','latex','FontSize',18);

set(gca,'Xticklabel',[],'layer','top')

subplot('position',[x0 y0 width height]); hold on; box on;
han1=plot(t,Var2,'k'); set(han1,'LineWidth',1);
han1=plot(t,VAR_GROWTH_RATE*t,'r--'); set(han1,'LineWidth',2);
han1=plot(t,0*t+VAR_THEORY,'b--'); set(han1,'LineWidth',2);

han1=legend('var(Y)','$\sigma^2 \frac{t}{\Delta t}$','$\sigma_\mathrm{max}^2=\frac{\alpha}{\alpha-1}\sigma^2$'); set(han1,'location','northeast','interpreter','latex','FontSize',20);
set(gca,'FontSize',16,'layer','top');
xlabel('time','interpreter','latex','FontSize',20); ylabel('var($Y$)','interpreter','latex','FontSize',20); 

axis([t0 tstop 0 MAX2]);

annotation(gcf,'textbox',[0.117 0.925 0.306 0.0475],'String',{'random walk + damping'},'FitBoxToText','off','linestyle','none','FontSize',18);

if PRINT
    print(gcf,'Fig_7_variance_growth.jpg','-djpeg','-r300');
end

figure(3);
set(gcf,'Color','w','PaperPositionMode','auto','Position',[100 100 1000 800]);

subplot('position',[x0 y0+height+ysep width height]);  hold on; box on;

sig_min=min(Y3(end,:)); % min Y at the end of the simulation
sig_max=max(Y3(end,:)); % max Y at the end of the simulation
cmap=flipud(jet(128)); % color map

for i=1:Nens
    sig=Y3(end,i); % Y at the end of the simulation
    c=round(interp1([sig_min sig_max],[1 128],sig)); % figure out the color of a specific line
    patchline(t,Y3(:,i),'linewidth',0.5,'edgealpha',0.4,'edgecolor',cmap(c,:)); % make a patchline for each ensemble member
end

han1=plot(t,0*t+3*UPPER2,'m--',t,0*t-3*UPPER2,'m--'); set(han1,'LineWidth',2);
set(gca,'FontSize',16,'Ytick',-MAX:2:MAX);
ylabel('$Y$','interpreter','latex','FontSize',20);
axis([t0 tstop -MAX MAX]);

han1=text(1,3*UPPER2+1,'$+3\sigma_{f,\mathrm{max}}$'); set(han1,'interpreter','latex','FontSize',18);
han1=text(1,-3*UPPER2-1.15,'$-3\sigma_{f,\mathrm{max}}$'); set(han1,'interpreter','latex','FontSize',18);

set(gca,'Xticklabel',[],'layer','top')

subplot('position',[x0 y0 width height]); hold on; box on;
han1=plot(t,Var3,'k'); set(han1,'LineWidth',1);
han1=plot(t,VAR_GROWTH_RATE2*t,'r--'); set(han1,'LineWidth',2);
%han1=plot(t,0*t+VAR_THEORY,'b--'); set(han1,'LineWidth',2);
han1=plot(t,0*t+VAR_THEORY2,'m--'); set(han1,'LineWidth',2);

han1=legend('var(Y)','$\left(\sigma^2+\frac{\Delta t}{\Delta T}\sigma_f^2\right)\frac{t}{\Delta t}$','$\sigma_{f,\mathrm{max}}^2=\frac{\alpha}{\alpha-1}\left(\sigma^2+\frac{\Delta t}{\Delta T}\sigma_f^2\right)$'); set(han1,'location','northeast','interpreter','latex','FontSize',20);
set(gca,'FontSize',16,'layer','top');
xlabel('time','interpreter','latex','FontSize',20); ylabel('var($Y$)','interpreter','latex','FontSize',20); 

axis([t0 tstop 0 MAX2]);

annotation(gcf,'textbox',[0.117 0.925 0.506 0.0475],'String',{'random walk + random forcing + damping'},'FitBoxToText','off','linestyle','none','FontSize',18);

if PRINT
    print(gcf,'Fig_8_variance_growth.jpg','-djpeg','-r300');
end

figure(4);
set(gcf,'Color','w','PaperPositionMode','auto','Position',[100 100 1000 800]);

subplot('position',[x0 y0+height+ysep width height]);  hold on; box on;

sig_min=min(Y4(end,:)); % min Y at the end of the simulation
sig_max=max(Y4(end,:)); % max Y at the end of the simulation
cmap=flipud(jet(128)); % color map

for i=1:Nens
    sig=Y4(end,i); % Y at the end of the simulation
    c=round(interp1([sig_min sig_max],[1 128],sig)); % figure out the color of a specific line
    patchline(t,Y4(:,i),'linewidth',0.5,'edgealpha',0.4,'edgecolor',cmap(c,:)); % make a patchline for each ensemble member
end

han1=plot(t,0*t+3*UPPER,'b--',t,0*t-3*UPPER,'b--'); set(han1,'LineWidth',2);
han1=plot(t,0*t+3*UPPER2,'m--',t,0*t-3*UPPER2,'m--'); set(han1,'LineWidth',2);

set(gca,'FontSize',16,'Ytick',-MAX:2:MAX);
ylabel('$Y$','interpreter','latex','FontSize',20);
axis([t0 tstop -MAX MAX]);

han1=text(1,3*UPPER+1,'$+3\sigma_\mathrm{max}$'); set(han1,'interpreter','latex','FontSize',18);
han1=text(1,-3*UPPER-1.15,'$-3\sigma_\mathrm{max}$'); set(han1,'interpreter','latex','FontSize',18);

han1=text(1,3*UPPER2+1,'$+3\sigma_{f,\mathrm{max}}$'); set(han1,'interpreter','latex','FontSize',18);
han1=text(1,-3*UPPER2-1.15,'$-3\sigma_{f,\mathrm{max}}$'); set(han1,'interpreter','latex','FontSize',18);

set(gca,'Xticklabel',[],'layer','top')

subplot('position',[x0 y0 width height]); hold on; box on;
han1=plot(t,Var4,'k'); set(han1,'LineWidth',1);
han1=plot(t,VAR_GROWTH_RATE*t,'r--'); set(han1,'LineWidth',2);
han1=plot(t,0*t+VAR_THEORY,'b--'); set(han1,'LineWidth',2);
han1=plot(t,0*t+VAR_THEORY2,'m--'); set(han1,'LineWidth',2);

han1=legend('var(Y)','$\sigma^2 \frac{t}{\Delta t}$','$\sigma_\mathrm{max}^2=\frac{\alpha}{\alpha-1}\sigma^2$','$\sigma_{f,\mathrm{max}}^2=\frac{\alpha}{\alpha-1}\left(\sigma^2+\frac{\Delta t}{\Delta T}\sigma_f^2\right)$'); set(han1,'Position',[0.577185185978259 0.338977105461743 0.352207602107298 0.19060622787159],'interpreter','latex','FontSize',20);
set(gca,'FontSize',16,'layer','top');
xlabel('time','interpreter','latex','FontSize',20); ylabel('var($Y$)','interpreter','latex','FontSize',20); 

axis([t0 tstop 0 MAX2]);

annotation(gcf,'textbox',[0.117 0.925 0.506 0.0475],'String',{'random walk + uniform forcing + damping'},'FitBoxToText','off','linestyle','none','FontSize',18);

if PRINT
    print(gcf,'Fig_9_variance_growth.jpg','-djpeg','-r300');
end

%%
figure(5);
set(gcf,'Color','w','PaperPositionMode','auto','Position',[100 100 1000 800]);

subplot('position',[x0 y0+height+ysep width height]);  hold on; box on;

sig_min=min(Y5(end,:)); % min Y at the end of the simulation
sig_max=max(Y5(end,:)); % max Y at the end of the simulation
cmap=flipud(jet(128)); % color map

for i=1:Nens
    sig=Y5(end,i); % Y at the end of the simulation
    c=round(interp1([sig_min sig_max],[1 128],sig)); % figure out the color of a specific line
    patchline(t,Y5(:,i),'linewidth',0.5,'edgealpha',0.4,'edgecolor',cmap(c,:)); % make a patchline for each ensemble member
end

han1=plot(t,0*t+3*UPPER,'b--',t,0*t-3*UPPER,'b--'); set(han1,'LineWidth',2);
han1=plot(t,0*t+3*UPPER2,'m--',t,0*t-3*UPPER2,'m--'); set(han1,'LineWidth',2);

% dahsed line %%%%%%%%
x = linspace(tstop/2,tstop/2, 400);
y = linspace(-MAX,MAX, 400);
yDashed = y;
yDashed(10:14:length(yDashed)) = NaN;
plot(x, yDashed,'k');

set(gca,'FontSize',16,'Ytick',-MAX:2:MAX);
ylabel('$Y$','interpreter','latex','FontSize',20);
axis([t0 tstop -MAX MAX]);

han1=text(1,3*UPPER+1,'$+3\sigma_\mathrm{max}$'); set(han1,'interpreter','latex','FontSize',18);
han1=text(1,-3*UPPER-1.15,'$-3\sigma_\mathrm{max}$'); set(han1,'interpreter','latex','FontSize',18);

han1=text(1,3*UPPER2+1,'$+3\sigma_{f,\mathrm{max}}$'); set(han1,'interpreter','latex','FontSize',18);
han1=text(1,-3*UPPER2-1.15,'$-3\sigma_{f,\mathrm{max}}$'); set(han1,'interpreter','latex','FontSize',18);

set(gca,'Xticklabel',[],'layer','top')

subplot('position',[x0 y0 width height]); hold on; box on;
han1=plot(t,Var5,'k'); set(han1,'LineWidth',1);
han1=plot(t,VAR_GROWTH_RATE*t,'r--'); set(han1,'LineWidth',2);
han1=plot(t,VAR_GROWTH_RATE2*(t-tstop/2+3),'g--'); set(han1,'LineWidth',2);
han1=plot(t,0*t+VAR_THEORY,'b--'); set(han1,'LineWidth',2);
han1=plot(t,0*t+VAR_THEORY2,'m--'); set(han1,'LineWidth',2);


% dahsed line %%%%%%%%
x = linspace(tstop/2,tstop/2, 400);
y = linspace(0,MAX2, 400);
yDashed = y;
yDashed(10:14:length(yDashed)) = NaN;
h=plot(x, yDashed,'k');
h.Annotation.LegendInformation.IconDisplayStyle = 'off';

axis([t0 tstop 0 MAX2]);

han1=legend('var(Y)','$\sigma^2 \frac{t}{\Delta t}$','$\left(\sigma^2+\frac{\Delta t}{\Delta T}\sigma_f^2\right)\frac{t}{\Delta t}$','$\sigma_\mathrm{max}^2=\frac{\alpha}{\alpha-1}\sigma^2$','$\sigma_{f,\mathrm{max}}^2=\frac{\alpha}{\alpha-1}\left(\sigma^2+\frac{\Delta t}{\Delta T}\sigma_f^2\right)$'); set(han1,'Position',[0.584185185978259 0.332310159547324 0.352207602107298 0.251440119700428],'interpreter','latex','FontSize',20);
set(gca,'FontSize',16,'layer','top');
xlabel('time','interpreter','latex','FontSize',20); ylabel('var($Y$)','interpreter','latex','FontSize',20); 

annotation(gcf,'doublearrow',[0.428000000000001 0.616000000000001],[0.9165 0.91625]);

annotation(gcf,'textbox',[0.342 0.91875 0.176 0.05625],'String',{'uniform forcing'},'FitBoxToText','off','linestyle','none','FontSize',18);
annotation(gcf,'textbox',[0.534 0.92625 0.199 0.0475],'String',{'ensemble forcing'},'FitBoxToText','off','linestyle','none','FontSize',18);

if PRINT
    print(gcf,'Fig_10_variance_growth.jpg','-djpeg','-r300');
end
