% variance and covariance of wave ensemble. SeanV.
close all; clc; %clear all;

Nens=1000; % the number of wave ensembles

RELOAD=1; % load in the ensemble wave data from Tairua
if RELOAD;
    
    %load('Results_100IndependentSimulations_3hourly.mat');
    load('Tairua_ensemble_waves_1000.mat');
    
    Nsim=size(Hs,2); % the number of wave ensembles
    
%     t=Simulation.time;  % time
%     Hs=Simulation.Hs;   % wave height
%     Tp=Simulation.Tm;   % wave period
%     Dir=Simulation.Dir; % wave direction
    
    if  Nens~=Nsim; % if the number of model ensembles is not the same as the number of wave ensebles  
        Hs=Hs(:,randi([1 Nsim],Nens,1));
        Tp=Tp(:,randi([1 Nsim],Nens,1));
        Dir=Dir(:,randi([1 Nsim],Nens,1));     
    end
    
end

Hs_bar=nanmean(Hs(:));            % mean wave height

DY=10;

f=-DY.*Hs./Hs_bar.*(Hs.^2-Hs_bar^2)./Hs_bar^2;  % forcing

%f=Hs.*(Hs.^2-Hs_bar^2)/Hs_bar^2;  % forcing

%plot(t,f); datetick('x');

len=length(t); % length of the wave time series

% find the factor, fac
if 0;
    
    N=2000; % based on first N elements in wave time series
    
    covt=NaN(N,N); % covariance matrix
    
    for i=1:N;      % for all N times
        for j=1:N;
            A=cov(f(i,:),f(j,:)); % calculate ... 
            covt(i,j)=A(2,1);     % and save covariance matrix   
        end
    end
    
    figure;
    pcolor(covt); shading flat; colormap(jet); colorbar; % plot covaraince matrix
    
    Svar=sum(diag(covt));     % get trace of covariance matrix
    A=covt-diag(diag(covt));  % remove diagonal elements from covaraince matrix
    Scov=sum(A(:));           % get sum of the off diagonal elements of the covaraince matrix
    
    R=Scov/Svar; % the ratio between the total covariance part and the variance part, which gives a scaling factor
    
    fac=1+R;
    
else
    fac=15;
end

dt=nanmean(diff(t));  % model time step

DT=28;   % model time scale
DY=10;   % model shoreline excursion parameter

lambda=1-dt/DT; % damping coefficient

Nsteps=20000;  % number of steps to run the model for

idst=1000;     % start id

t0=t(idst);            % the inital time
t_mod=t(idst:idst+Nsteps-1,:)-t0; % truncate the time series to only run the model for Nsteps
f_mod=f(idst:idst+Nsteps-1,:);    % truncate the forcing time series to only run the model for Nsteps

ti=t_mod(1);       % initial time for plotting
tstop=t_mod(end);  % final time for plotting

sigmaf=sqrt(nanmean(var(f_mod,[],2)));  % calculate the mean standard deviation of the forcing function

sigma=0.2;  % the standard devation of the added noise

NOISE=sigma/dt;  % the added noise based on sigma

VAR_GROWTH_RATE=(sigma^2+fac*(dt/DT)^2*sigmaf^2)/dt;

a=1/lambda.^2;   % the a coefficient based on lambda

VAR_THEORY    =a/(a-1)*(sigma^2+fac*(dt/DT).^2*sigmaf^2); % the theoretical (yet approximate) upper bound for the model variance

UPPER=sqrt(VAR_THEORY); % the standard deviation based on the theoretical varaince

MIN=-180;
MAX=8*UPPER;            % upper limit for model state for plotting
MAX2=3*VAR_THEORY;    % upper limit for model varaince for plotting

% RUN FORWARD MODEL
Y=NaN(Nsteps,Nens); Var=NaN(Nsteps,1);
Y(1,:)=0;           Var(1)=var( Y(1,:));

for n=1:Nsteps-1;

    % forward model, real forcing
    Y(n+1,:)=lambda*Y(n,:)+dt/DT*f_mod(n,:)+dt*NOISE*randn(1,Nens);
    
    % variance
    Var(n+1)=var(Y(n+1,:));

end

PRINT=1;

x0=0.1;
y0=0.08;
width=0.80;
height=0.43;
ysep=0.04;

%% Generate fig

figure(1);
set(gcf,'Color','w','PaperPositionMode','auto','Position',[100 100 1000 800]);

subplot('position',[x0 y0+height+ysep width height]);  hold on; box on;

sig_min=-50; % min Y at the end of the simulation
sig_max=10; % max Y at the end of the simulation
cmap=flipud(jet(128)); % color map

for i=1:Nens
    sig=Y(end,i); % Y at the end of the simulation
    
    c=round(interp1([sig_min sig_max],[1 128],sig)); % figure out the color of a specific line
    
    if sig>sig_max
        c=128;
    elseif sig<sig_min
        c=1;
    end
    
    patchline(t_mod,Y(:,i),'linewidth',0.5,'edgealpha',0.4,'edgecolor',cmap(c,:)); % make a patchline for each ensemble member
end

han1=plot(t_mod,0*t_mod+3*UPPER,'k--',t_mod,0*t_mod-3*UPPER,'k--'); set(han1,'LineWidth',2);
set(gca,'FontSize',16,'Ytick',MIN:20:MAX);
%xlabel('time','interpreter','latex'); 
ylabel('$Y [m]$','interpreter','latex','FontSize',20);
axis([ti tstop MIN MAX]);

han1=text(525,3*UPPER+10.2,'$+3\sigma_{F,\mathrm{max}}$'); set(han1,'interpreter','latex','FontSize',18);
han1=text(384.587398897059, -156.904681929074,'$-3\sigma_{F,\mathrm{max}}$'); set(han1,'interpreter','latex','FontSize',18);

annotation(gcf,'arrow',[0.242 0.216],[0.60375 0.785]);

set(gca,'Xticklabel',[],'layer','top')

subplot('position',[x0 y0 width height]); hold on; box on;
han1=plot(t_mod,Var,'k'); set(han1,'LineWidth',1);
han1=plot(t_mod,VAR_GROWTH_RATE*t_mod,'r--'); set(han1,'LineWidth',2);
han1=plot(t_mod,0*t_mod+VAR_THEORY,'b--'); set(han1,'LineWidth',2);

han1=legend('var(Y)','$\left(\sigma^2+F\frac{\Delta t}{\Delta T}\sigma_f^2\right)\frac{t}{\Delta t}$','$\sigma^2_{F,\mathrm{max}}=\frac{\alpha}{\alpha-1}\left(\sigma^2+F\frac{\Delta t}{\Delta T}\sigma_f^2\right)$'); set(han1,'Position',[0.498607166987444 0.317498884201082 0.378059499679224 0.16833444913225],'interpreter','latex','FontSize',20);
set(gca,'FontSize',16,'layer','top');
xlabel('time [days]','interpreter','latex','FontSize',20); ylabel('var($Y$)','interpreter','latex','FontSize',20); 
annotation(gcf,'textbox',[0.117 0.90375 0.632 0.06875],'String',{'Idealized Shoreline Change Model Ensemble'},'FitBoxToText','off','linestyle','none','FontSize',18);

axis([ti tstop 0 MAX2]);

if PRINT
    print(gcf,'Fig_11_variance_growth_NEW_WAVES_test_long.jpg','-djpeg','-r300');
end